@@ -3,9 +3,9 @@ use crate::components::{
     Arsc, Config, Header, Package, ResourceEntry, ResourceValue, Spec, Specs, StringPool, Type,
     Value,
 };
-use crate::writer::components_sizing::{padding, ByteSizing};
+use crate::writer::components_sizing::{padding, ByteSizing, ConstByteSizing};
 use crate::writer::with_header::WithHeader;
-use crate::Style;
+use crate::{Resources, Style, StyleSpan};
 use std::io::{Result, Write};
 
 /// types that implement this trait should define the function
@@ -98,8 +98,8 @@ impl ArscSerializable for StringPool {
             position += style.write(output)?;
         }
         if !self.styles.is_empty() {
-            position += write_util::write_i32(output, -1)?;
-            position += write_util::write_i32(output, -1)?;
+            position += write_util::write_u32(output, u32::MAX)?;
+            position += write_util::write_u32(output, u32::MAX)?;
         }
 
         Ok(position)
@@ -117,10 +117,12 @@ impl StringPool {
 
     fn write_style_offsets<W: Write>(&self, output: &mut W) -> Result<usize> {
         let mut position = 0;
-        for index in 0..self.styles.len() {
-            position += write_util::write_u32(output, index * 16)?;
+        let mut written = 0;
+        for style in &self.styles {
+            written += write_util::write_u32(output, position)?;
+            position += style.spans.len() * StyleSpan::SIZE + 4;
         }
-        Ok(position)
+        Ok(written)
     }
 
     fn write_utf8_offsets<W: Write>(&self, output: &mut W) -> Result<usize> {
@@ -190,11 +192,21 @@ impl StringPool {
 }
 
 impl ArscSerializable for Style {
+    fn write<W: Write>(&self, output: &mut W) -> Result<usize> {
+        let mut written = 0;
+        for span in &self.spans {
+            written += span.write(output)?;
+        }
+        written += write_util::write_u32(output, u32::MAX)?;
+        Ok(written)
+    }
+}
+
+impl ArscSerializable for StyleSpan {
     fn write<W: Write>(&self, output: &mut W) -> Result<usize> {
         let mut position = write_util::write_u32(output, self.name)?;
         position += write_util::write_u32(output, self.start)?;
         position += write_util::write_u32(output, self.end)?;
-        position += write_util::write_i32(output, -1)?;
         Ok(position)
     }
 }
@@ -225,37 +237,48 @@ impl ArscSerializable for Config {
         position += write_util::write_u8(output, self.type_id)?;
         position += write_util::write_u8(output, self.res0)?;
         position += write_util::write_u16(output, self.res1)?;
-        position += write_util::write_u32(output, self.entry_count)?;
-        let entry_start =
-            position + 4 + self.id.len() + padding(self.id.len()) + self.entry_count * 4;
+        position += write_util::write_u32(output, self.resources.entry_count())?;
+        let entry_start = position
+            + 4
+            + self.id.len()
+            + padding(self.id.len())
+            + self.resources.entry_count() * 4;
         position += write_util::write_u32(output, entry_start)?;
         position += output.write(&self.id)?;
         position += output.write(&vec![0; padding(self.id.len())])?;
-        position += self.write_resources(output)?;
+        position += self.resources.write(output)?;
 
         Ok(position)
     }
 }
 
-impl Config {
-    fn write_resources<W: Write>(&self, output: &mut W) -> Result<usize> {
+impl ArscSerializable for Resources {
+    fn write<W: Write>(&self, output: &mut W) -> Result<usize> {
         let mut position = self.write_entries(output)?;
-        for resource in self.resources.values() {
+        for resource in &self.resources {
             position += resource.write(output)?;
         }
         Ok(position)
     }
+}
 
+impl Resources {
     fn write_entries<W: Write>(&self, output: &mut W) -> Result<usize> {
         let mut offset = 0;
         let mut written = 0;
-        for entry in 0..self.entry_count {
-            if let Some(resource) = self.resources.get(&entry) {
-                written += write_util::write_i32(output, offset)?;
-                offset += 2 + resource.size(); // _size + resource
-            } else {
-                written += write_util::write_i32(output, -1)?;
+
+        let mut expected_spec_id = 0;
+        for resource in &self.resources {
+            let gaps = resource.spec_id - expected_spec_id;
+            for _ in 0..gaps {
+                written += write_util::write_u32(output, u32::MAX)?;
             }
+            written += write_util::write_u32(output, offset)?;
+            offset += resource.size();
+            expected_spec_id = resource.spec_id + 1;
+        }
+        for _ in expected_spec_id..self.entry_count() {
+            written += write_util::write_u32(output, u32::MAX)?;
         }
         Ok(written)
     }
@@ -286,10 +309,10 @@ impl ArscSerializable for Package {
 
         let type_string_offset = position + 5 * 4;
         position += write_util::write_u32(output, type_string_offset)?; // type_string_offset
-        position += write_util::write_u32(output, 0)?; // last_public_type
+        position += write_util::write_u32(output, self.last_public_type)?; // last_public_type
         let key_string_offset = type_string_offset + self.type_names.size();
         position += write_util::write_u32(output, key_string_offset)?; // key_string_offset
-        position += write_util::write_u32(output, 0)?; // last_public_key
+        position += write_util::write_u32(output, self.last_public_key)?; // last_public_key
         position += write_util::write_u32(output, 0)?; // type_id_offset
 
         position += self.type_names.write(output)?;
